/*
**  ObjCHandler.m
**
**  Copyright (c) 2003
**
**  Author: Yen-Ju  <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ObjCHandler.h"
#include "GNUstep.h"
#include <AppKit/AppKit.h>

@implementation ObjCHandler

#define STRCMP(str) (!strcmp(cstr, str))

- (void) string: (NSString *) element
{
  const char *cstr = [element cString];

  [super string: element];

  if (_commentType != NoComment)
    {
      /* Within comment */
    }
  else if (_stringBegin == YES)
    {
      /* Within string */
    }
  else if (_preChar == '@')
    {
      if ((STRCMP("interface")) && (situation == CodeHandlerOtherSituation))
        {
          situation = CodeHandlerInterfaceBegin;
          NSLog(@"@interface");
        }
      else if (STRCMP("end"))
        {
          if ((situation == CodeHandlerInterfaceBegin) ||
              (situation == CodeHandlerDeclarationEnd))
            {
               situation = CodeHandlerInterfaceEnd;
               NSLog(@"@end");
 	    }
        }
    }
  else if (situation == CodeHandlerDeclarationBegin)
    {
      /* Only care within declaration */
      if (STRCMP("IBOutlet"))
        {
          [currentClass setString: element];
        }
      else if (STRCMP("id"))
        {
	  if ([currentClass length])
	    [currentClass appendString: @" "];

          [currentClass appendString: element];
        }
      /* Other type */
      else if ( 
                STRCMP("int") || STRCMP("long") || STRCMP("short") ||
                STRCMP("char") || STRCMP("float") || STRCMP("double") ||
                STRCMP("void") || STRCMP("union") || STRCMP("unichar") ||
                STRCMP("const") || STRCMP("signed") || STRCMP("unsigned") ||
                STRCMP("static") || STRCMP("volatile") ||
                STRCMP("enum") || (STRCMP("BOOL"))
               )
        {
	  /* Not valid outlet */
	  [currentClass setString: @""];
        }
      else if (([currentClass isEqualToString: @"IBOutlet"]) && (situation == CodeHandlerDeclarationBegin))
        {
	  /* This should be a class name */
          [currentClass appendString: @" "];
	  [currentClass appendString: element];
        }
      else if (([currentClass length]) && (situation == CodeHandlerDeclarationBegin))
        {
	  if ([currentClass hasSuffix: @"id"] == NO)
	    [currentVariable setString: @"*"];

          [currentVariable appendString: element];
        }
    }

  _preChar = 0;
}

- (void) number: (NSString *) element 
{
  [super number: element];
  _preChar = 0;
}

- (void) spaceAndNewLine: (unichar) element 
{
  [super spaceAndNewLine: element];
  _preChar = element;
}

- (void) symbol: (unichar) element 
{
  [super symbol: element];

  if ( ((_preChar == '/') && ((element == '*') || (element == '/'))) ||
       ((_preChar == '*') && (element == '/'))
     )
    {
      /* symbol of comment */
    }
  else if ((element == '{') && (situation == CodeHandlerInterfaceBegin))
    {
      situation = CodeHandlerDeclarationBegin;
      NSLog(@"{");
    }
  else if ((element == '}') && (situation == CodeHandlerDeclarationBegin))
    {
      situation = CodeHandlerDeclarationEnd;
      NSLog(@"}");
    }
  else if ((element == ';') && (situation == CodeHandlerDeclarationBegin))
    {
      if (([currentClass length]) && ([currentVariable length]))
	{
	  NSLog(@"\t%@ %@", currentClass, currentVariable);
	}
      [currentClass setString: @""];
      [currentVariable setString: @""];
    }
  else if ((element == ',') && (situation == CodeHandlerDeclarationBegin))
    {
      if (([currentClass length]) && ([currentVariable length]))
	{
	  NSLog(@"\t%@ %@", currentClass, currentVariable);
	}
      [currentVariable setString: @""];
    }
  _preChar = element;
}

- (void) invisible: (unichar) element
{
  [super invisible: element];
  _preChar = element;
}

- (id) init
{
  self = [super init];
  situation = CodeHandlerOtherSituation;

  currentClass = [[NSMutableString alloc] init];
  currentVariable = [[NSMutableString alloc] init];

  return self;
}

- (void) dealloc
{
  RELEASE(currentClass);
  RELEASE(currentVariable);

  [super dealloc];
}

@end

